// This file is a part of the Framsticks GDK library.
// Copyright (C) 2002-2011  Szymon Ulatowski.  See LICENSE.txt for details.
// Refer to http://www.framsticks.com/ for further information.

#include "paramvmobj.h"
#include "collectionobj.h"

#define FIELDSTRUCT ParamVMObj
static ParamEntry paramvmobj_paramtab[]=
{
{"Interface",1,19,"Interface","Used to query for object member descriptions. Example:\nvar iface=Interface.makeFrom(someobject); var description=\"this object has \"+iface.properties+\" properties, first property is \"+iface.getId(0);"},
{"name",0,2+PARAM_NOSTATIC,"object name","s",GETONLY(name),},
{"properties",0,2+PARAM_NOSTATIC,"property count","d",GETONLY(props),},
{"groups",0,2+PARAM_NOSTATIC,"group count","d",GETONLY(groups),},
{"get",0,PARAM_NOSTATIC,"value of item #","p x(d index)",PROCEDURE(p_get),},
{"getId",0,PARAM_NOSTATIC,"id for item #","p s(d index)",PROCEDURE(p_getid),},
{"getName",0,PARAM_NOSTATIC,"name for item #","p s(d index)",PROCEDURE(p_getname),},
{"getGroup",0,PARAM_NOSTATIC,"group # for item #","p d(d index)",PROCEDURE(p_getgroup),},
{"getGroupName",0,PARAM_NOSTATIC,"group name for group #","p s(d index)",PROCEDURE(p_getgrname),},
{"getType",0,PARAM_NOSTATIC,"type for item #","p s(d index)",PROCEDURE(p_gettype),},
{"getFlags",0,PARAM_NOSTATIC,"flags for item #","p d(d index)",PROCEDURE(p_getflags),},
{"getHelp",0,PARAM_NOSTATIC,"help for item #","p s(d index)",PROCEDURE(p_gethelp),},
{"findId",0,PARAM_NOSTATIC,"item # for id","p d(s name)",PROCEDURE(p_findid),},
{"findGroupId",0,PARAM_NOSTATIC,"group # for group name","p d(s name)",PROCEDURE(p_findgroupid),},
{"findIdInGroup",0,PARAM_NOSTATIC,"item # for id in group","p d(s name,x group name or index)",PROCEDURE(p_findidingroup),},
{"makeFrom",0,2,"get interface object","p oInterface(o)",PROCEDURE(p_make),},
{"set",0,PARAM_NOSTATIC,"set value of item #","p(d index,x value)",PROCEDURE(p_set),},
{"setDefault",0,PARAM_NOSTATIC,"set default value for item #","p(d index)",PROCEDURE(p_setDef),},
{"setAllDefault",0,PARAM_NOSTATIC,"set default values for all items","p()",PROCEDURE(p_setAllDef),},
{"invoke",0,PARAM_NOSTATIC,"invoke action","p(x function name or index,oVector arguments)",PROCEDURE(p_call)},
{0,0,0,},
};

#undef FIELDSTRUCT

ParamVMObj::ParamVMObj(ParamInterface *pi):iface(pi),par(paramvmobj_paramtab,this)
{}

ExtObject ParamVMObj::makeObject(ParamInterface *pi)
{
ParamVMObj *o=new ParamVMObj(pi);
return ExtObject(&o->par,o);
}

void ParamVMObj::p_make(ExtValue *args,ExtValue *ret)
{
ParamInterface *pi=args->getObject().getParamInterface();
if (pi)
	ret->setObject(makeObject(pi));
else
	ret->setEmpty();
}

int ParamVMObj::findGroupId(const char* name)
{
for(int i=0;i<iface->getGroupCount();i++)
	if (!strcmp(iface->grname(i),name))
		return i;
return -1;
}

void ParamVMObj::p_findgroupid(ExtValue *args,ExtValue *ret)
{
const SString &s=args[0].getString();
ret->setInt(findGroupId((const char*)s));
}

void ParamVMObj::p_findidingroup(ExtValue *args,ExtValue *ret)
{
ExtValue& v=args[0];
const SString& s=args[1].getString();
int g;
if (v.type==TInt)
	g=v.getInt();
else
	g=findGroupId((const char*)v.getString());
if (g<0)
	{ret->setInt(-1);return;}
for(int i=0;i<iface->getPropCount();i++)
	if (iface->group(i)==g)
		if (!strcmp(iface->id(i),s))
			{ret->setInt(i);return;}
ret->setInt(-1);
}

void ParamVMObj::p_call(ExtValue *args,ExtValue *ret)
{
if (!iface) return;
ExtValue& prop=args[1];
int p;
if (prop.type==TInt) p=prop.getInt(); else p=iface->findId(prop.getString());
if (p<0) return;
const ExtValue& a=args[0];
const ExtObject& o=a.getObject();
ParamInterface *argstype=o.getParamInterface();
ExtValue *useargs=0;
VectorObject* v;
ExtValue *tmpmem=0;
int n=0;
if (argstype && (argstype==&VectorObject::par))
	{
	v=(VectorObject*)o.getTarget();
	n=v->data.size();
	}
if (n)
	{
	tmpmem=new ExtValue[n];
	useargs=tmpmem;
	for(int i=0;i<n;i++)
		tmpmem[i]=*(v->get(i));
	}
iface->call(p,useargs,ret);
if (n)
	delete[] useargs;
}

